@@ -24,8 +24,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
 
 #include "core.h"
 #include "print.h"
@@ -558,7 +558,7 @@ int mdc800_setFlashLight (Camera* camera,int value)
 		printCoreError ("(mdc800_setFlashLight) sending command fails.\n");
 		return ret;
 	}
-	printCoreNote (mdc800_getFlashLightString(value));
+	printCoreNote ("%s", mdc800_getFlashLightString(value));
 	printCoreNote ("\n");
 	return GP_OK;
 }
