@@ -32,6 +32,15 @@ htod32ap (PTPParams *params, unsigned char *a, uint32_t val)
 		htobe32a(a,val);
 }
 
+static inline void
+htod64ap (PTPParams *params, unsigned char *a, uint64_t val)
+{
+	if (params->byteorder==PTP_DL_LE)
+		htole64a(a,val);
+	else 
+		htobe64a(a,val);
+}
+
 static inline uint16_t
 dtoh16p (PTPParams *params, uint16_t var)
 {
@@ -44,6 +53,12 @@ dtoh32p (PTPParams *params, uint32_t var)
 	return ((params->byteorder==PTP_DL_LE)?le32toh(var):be32toh(var));
 }
 
+static inline uint64_t
+dtoh64p (PTPParams *params, uint64_t var)
+{
+	return ((params->byteorder==PTP_DL_LE)?le64toh(var):be64toh(var));
+}
+
 static inline uint16_t
 dtoh16ap (PTPParams *params, unsigned char *a)
 {
@@ -59,24 +74,16 @@ dtoh32ap (PTPParams *params, unsigned char *a)
 static inline uint64_t
 dtoh64ap (PTPParams *params, unsigned char *a)
 {
-	uint64_t tmp = 0;
-	int i;
-
-	if (params->byteorder==PTP_DL_LE) {
-		for (i=0;i<8;i++)
-			tmp |= (((uint64_t)a[i]) << (8*i));
-	} else {
-		for (i=0;i<8;i++)
-			tmp |= (((uint64_t)a[i]) << (8*(7-i)));
-	}
-	return tmp;
+	return ((params->byteorder==PTP_DL_LE)?le64atoh(a):be64atoh(a));
 }
 
 #define htod8a(a,x)	*(uint8_t*)(a) = x
 #define htod16a(a,x)	htod16ap(params,a,x)
 #define htod32a(a,x)	htod32ap(params,a,x)
+#define htod64a(a,x)	htod64ap(params,a,x)
 #define htod16(x)	htod16p(params,x)
 #define htod32(x)	htod32p(params,x)
+#define htod64(x)	htod64p(params,x)
 
 #define dtoh8a(x)	(*(uint8_t*)(x))
 #define dtoh16a(a)	dtoh16ap(params,a)
@@ -84,46 +91,52 @@ dtoh64ap (PTPParams *params, unsigned char *a)
 #define dtoh64a(a)	dtoh64ap(params,a)
 #define dtoh16(x)	dtoh16p(params,x)
 #define dtoh32(x)	dtoh32p(params,x)
+#define dtoh64(x)	dtoh64p(params,x)
 
 
 static inline char*
 ptp_unpack_string(PTPParams *params, unsigned char* data, uint16_t offset, uint8_t *len)
 {
-	int i;
-	uint8_t loclen;
+	uint8_t length;
+	uint16_t string[PTP_MAXSTRLEN+1];
+	/* allow for UTF-8: max of 3 bytes per UCS-2 char, plus final null */
+	char loclstr[PTP_MAXSTRLEN*3+1]; 
+	size_t nconv, srclen, destlen;
+	char *src, *dest;
 
-	/* Cannot exceed 255 (PTP_MAXSTRLEN) since it is a single byte, duh ... */
-	loclen = dtoh8a(&data[offset]);
-	/* This len is used to advance the buffer pointer */
-	*len = loclen;
-	if (loclen) {
-		uint16_t string[PTP_MAXSTRLEN+1];
-		char *stringp = (char *) string;
-		char loclstr[PTP_MAXSTRLEN*3+1]; /* UTF-8 encoding is max 3 bytes per UCS2 char. */
-		char *locp = loclstr;
-		size_t nconv;
-		size_t convlen = loclen * 2; /* UCS-2 is 16 bit wide */
-		size_t convmax = PTP_MAXSTRLEN*3;
-		
-		for (i=0;i<loclen;i++) {
-			string[i]=dtoh16a(&data[offset+i*2+1]);
+	length = dtoh8a(&data[offset]);	/* PTP_MAXSTRLEN == 255, 8 bit len */
+	*len = length;
+	if (length == 0)		/* nothing to do? */
+		return(NULL);
+
+	/* copy to string[] to ensure correct alignment for iconv(3) */
+	memcpy(string, &data[offset+1], length * sizeof(string[0]));
+	string[length] = 0x0000U;   /* be paranoid!  add a terminator. */
+	loclstr[0] = '\0';
+    
+	/* convert from camera UCS-2 to our locale */
+	src = (char *)string;
+	srclen = length * sizeof(string[0]);
+	dest = loclstr;
+	destlen = sizeof(loclstr)-1;
+	nconv = iconv(params->cd_ucs2_to_locale, &src, &srclen, 
+			&dest, &destlen);
+	if (nconv == (size_t) -1) { /* do it the hard way */
+		int i;
+		/* try the old way, in case iconv is broken */
+		for (i=0;i<length;i++) {
+			if (dtoh16a(&data[offset+1+2*i])>127)
+				loclstr[i] = '?';
+			else
+				loclstr[i] = dtoh16a(&data[offset+1+2*i]);
 		}
-		/* be paranoid! Add a terminator. :( */
-		string[loclen]=0x0000U;
-		loclstr[0]='\0';
-		/* loclstr=ucs2_to_utf8(string); */
-		/* Do the conversion.  */
-		nconv = iconv (params->cd_ucs2_to_locale, &stringp, &convlen, &locp, &convmax);
-		/* FIXME: handle size errors */
-		loclstr[PTP_MAXSTRLEN*3] = '\0';
-		if (nconv == (size_t) -1)
-			return NULL;
-		return strdup(loclstr);
+		dest = loclstr+length;
 	}
-	return NULL;
+	*dest = '\0';
+	loclstr[sizeof(loclstr)-1] = '\0';   /* be safe? */
+	return(strdup(loclstr));
 }
 
-
 static inline int
 ucs2strlen(uint16_t const * const unicstr)
 {
@@ -138,7 +151,6 @@ ucs2strlen(uint16_t const * const unicstr)
 static inline void
 ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t offset, uint8_t *len)
 {
-	int i;
 	int packedlen;
 	uint16_t ucs2str[PTP_MAXSTRLEN+1];
 	char *ucs2strp = (char *) ucs2str;
@@ -148,11 +160,15 @@ ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t o
 	size_t convmax = PTP_MAXSTRLEN * 2; /* Includes the terminator */
 
 	/* Cannot exceed 255 (PTP_MAXSTRLEN) since it is a single byte, duh ... */
-	ucs2str[0] = 0x0000U;
-	memset(ucs2strp, 0, PTP_MAXSTRLEN*2+2);
-	nconv = iconv (params->cd_locale_to_ucs2, &stringp, &convlen, &ucs2strp, &convmax);
+	memset(ucs2strp, 0, sizeof(ucs2str));  /* XXX: necessary? */
+	nconv = iconv(params->cd_locale_to_ucs2, &stringp, &convlen,
+		&ucs2strp, &convmax);
 	if (nconv == (size_t) -1)
 		ucs2str[0] = 0x0000U;
+	/*
+	 * XXX: isn't packedlen just ( (uint16_t *)ucs2strp - ucs2str )?
+	 *      why do we need ucs2strlen()?
+	 */
 	packedlen = ucs2strlen(ucs2str);
 	if (packedlen > PTP_MAXSTRLEN-1) {
 		*len=0;
@@ -161,10 +177,8 @@ ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t o
 	
 	/* number of characters including terminating 0 (PTP standard confirmed) */
 	htod8a(&data[offset],packedlen+1);
-	for (i=0;i<packedlen && i< PTP_MAXSTRLEN; i++) {
-		htod16a(&data[offset+i*2+1],ucs2str[i]);
-	}
-	htod16a(&data[offset+i*2+1],0x0000);
+	memcpy(&data[offset+1], &ucs2str[0], packedlen * sizeof(ucs2str[0]));
+	htod16a(&data[offset+packedlen*2+1], 0x0000);  /* terminate 0 */
 
 	/* The returned length is in number of characters */
 	*len = (uint8_t) packedlen+1;
@@ -176,8 +190,12 @@ ptp_get_packed_stringcopy(PTPParams *params, char *string, uint32_t *packed_size
 	uint8_t packed[PTP_MAXSTRLEN*2+3], len;
 	size_t plen;
 	unsigned char *retcopy = NULL;
-
-	ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
+  
+	if (string == NULL)
+	  ptp_pack_string(params, "", (unsigned char*) packed, 0, &len);
+	else
+	  ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
+  
 	/* returned length is in characters, then one byte for string length */
 	plen = len*2 + 1;
 	
@@ -294,6 +312,20 @@ ptp_unpack_DI (PTPParams *params, unsigned char* data, PTPDeviceInfo *di, unsign
 		PTP_di_OperationsSupported+totallen,
 		&len);
 }
+
+static void
+ptp_free_DI (PTPDeviceInfo *di) {
+	if (di->SerialNumber) free (di->SerialNumber);
+	if (di->DeviceVersion) free (di->DeviceVersion);
+	if (di->Model) free (di->Model);
+	if (di->Manufacturer) free (di->Manufacturer);
+	if (di->ImageFormats) free (di->ImageFormats);
+	if (di->CaptureFormats) free (di->CaptureFormats);
+	if (di->VendorExtensionDesc) free (di->VendorExtensionDesc);
+	if (di->OperationsSupported) free (di->OperationsSupported);
+	if (di->EventsSupported) free (di->EventsSupported);
+	if (di->DevicePropertiesSupported) free (di->DevicePropertiesSupported);
+}
 	
 /* ObjectHandles array pack/unpack */
 
@@ -302,7 +334,12 @@ ptp_unpack_DI (PTPParams *params, unsigned char* data, PTPDeviceInfo *di, unsign
 static inline void
 ptp_unpack_OH (PTPParams *params, unsigned char* data, PTPObjectHandles *oh, unsigned int len)
 {
-	oh->n = ptp_unpack_uint32_t_array(params, data, PTP_oh, &oh->Handler);
+	if (len) {
+		oh->n = ptp_unpack_uint32_t_array(params, data, PTP_oh, &oh->Handler);
+	} else {
+		oh->n = 0;
+		oh->Handler = NULL;
+	} 
 }
 
 /* StoreIDs array pack/unpack */
@@ -364,19 +401,23 @@ ptp_unpack_SI (PTPParams *params, unsigned char* data, PTPStorageInfo *si, unsig
 #define PTP_oi_filenamelen		52
 #define PTP_oi_Filename			53
 
+/* the max length assuming zero length dates. We have need 3 */
+/* bytes for these. */
+#define PTP_oi_MaxLen PTP_oi_Filename+(PTP_MAXSTRLEN+1)*2+3
+
 static inline uint32_t
 ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, unsigned char** oidataptr)
 {
 	unsigned char* oidata;
 	uint8_t filenamelen;
 	uint8_t capturedatelen=0;
-	/* let's allocate some memory first; XXX i'm sure it's wrong */
-	oidata=malloc(PTP_oi_Filename+(strlen(oi->Filename)+1)*2+4);
+	/* let's allocate some memory first; correct assuming zero length dates */
+	oidata=malloc(PTP_oi_MaxLen);
 	/* the caller should free it after use! */
 #if 0
 	char *capture_date="20020101T010101"; /* XXX Fake date */
 #endif
-	memset (oidata, 0, (PTP_oi_Filename+(strlen(oi->Filename)+1)*2+4));
+	memset (oidata, 0, PTP_oi_MaxLen);
 	htod32a(&oidata[PTP_oi_StorageID],oi->StorageID);
 	htod16a(&oidata[PTP_oi_ObjectFormat],oi->ObjectFormat);
 	htod16a(&oidata[PTP_oi_ProtectionStatus],oi->ProtectionStatus);
@@ -423,7 +464,45 @@ ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, unsigned char** oidataptr)
 	/* XXX this function should return dataset length */
 	
 	*oidataptr=oidata;
-	return (PTP_oi_Filename+(filenamelen+1)*2+(capturedatelen+1)*4);
+	return (PTP_oi_Filename+filenamelen*2+(capturedatelen+1)*3);
+}
+
+static time_t
+ptp_unpack_PTPTIME (const char *str) {
+	char ptpdate[40];
+	char tmp[5];
+	int  ptpdatelen;
+	struct tm tm;
+
+	if (!str)
+		return 0;
+	ptpdatelen = strlen(str);
+	if (ptpdatelen >= sizeof (ptpdate))
+		return 0;
+	strcpy (ptpdate, str);
+	if (ptpdatelen<=15)
+		return 0;
+
+	memset(&tm,0,sizeof(tm));
+	strncpy (tmp, ptpdate, 4);
+	tmp[4] = 0;
+	tm.tm_year=atoi (tmp) - 1900;
+	strncpy (tmp, ptpdate + 4, 2);
+	tmp[2] = 0;
+	tm.tm_mon = atoi (tmp) - 1;
+	strncpy (tmp, ptpdate + 6, 2);
+	tmp[2] = 0;
+	tm.tm_mday = atoi (tmp);
+	strncpy (tmp, ptpdate + 9, 2);
+	tmp[2] = 0;
+	tm.tm_hour = atoi (tmp);
+	strncpy (tmp, ptpdate + 11, 2);
+	tmp[2] = 0;
+	tm.tm_min = atoi (tmp);
+	strncpy (tmp, ptpdate + 13, 2);
+	tmp[2] = 0;
+	tm.tm_sec = atoi (tmp);
+	return mktime (&tm);
 }
 
 static inline void
@@ -432,10 +511,6 @@ ptp_unpack_OI (PTPParams *params, unsigned char* data, PTPObjectInfo *oi, unsign
 	uint8_t filenamelen;
 	uint8_t capturedatelen;
 	char *capture_date;
-	char tmp[16];
-	struct tm tm;
-
-	memset(&tm,0,sizeof(tm));
 
 	oi->StorageID=dtoh32a(&data[PTP_oi_StorageID]);
 	oi->ObjectFormat=dtoh16a(&data[PTP_oi_ObjectFormat]);
@@ -459,56 +534,14 @@ ptp_unpack_OI (PTPParams *params, unsigned char* data, PTPObjectInfo *oi, unsign
 	/* subset of ISO 8601, without '.s' tenths of second and 
 	 * time zone
 	 */
-	if (capturedatelen>15)
-	{
-		strncpy (tmp, capture_date, 4);
-		tmp[4] = 0;
-		tm.tm_year=atoi (tmp) - 1900;
-		strncpy (tmp, capture_date + 4, 2);
-		tmp[2] = 0;
-		tm.tm_mon = atoi (tmp) - 1;
-		strncpy (tmp, capture_date + 6, 2);
-		tmp[2] = 0;
-		tm.tm_mday = atoi (tmp);
-		strncpy (tmp, capture_date + 9, 2);
-		tmp[2] = 0;
-		tm.tm_hour = atoi (tmp);
-		strncpy (tmp, capture_date + 11, 2);
-		tmp[2] = 0;
-		tm.tm_min = atoi (tmp);
-		strncpy (tmp, capture_date + 13, 2);
-		tmp[2] = 0;
-		tm.tm_sec = atoi (tmp);
-		oi->CaptureDate=mktime (&tm);
-	}
+	oi->CaptureDate = ptp_unpack_PTPTIME(capture_date);
 	free(capture_date);
 
-	/* now it's modification date ;) */
+	/* now the modification date ... */
 	capture_date = ptp_unpack_string(params, data,
 		PTP_oi_filenamelen+filenamelen*2
 		+capturedatelen*2+2,&capturedatelen);
-	if (capturedatelen>15)
-	{
-		strncpy (tmp, capture_date, 4);
-		tmp[4] = 0;
-		tm.tm_year=atoi (tmp) - 1900;
-		strncpy (tmp, capture_date + 4, 2);
-		tmp[2] = 0;
-		tm.tm_mon = atoi (tmp) - 1;
-		strncpy (tmp, capture_date + 6, 2);
-		tmp[2] = 0;
-		tm.tm_mday = atoi (tmp);
-		strncpy (tmp, capture_date + 9, 2);
-		tmp[2] = 0;
-		tm.tm_hour = atoi (tmp);
-		strncpy (tmp, capture_date + 11, 2);
-		tmp[2] = 0;
-		tm.tm_min = atoi (tmp);
-		strncpy (tmp, capture_date + 13, 2);
-		tmp[2] = 0;
-		tm.tm_sec = atoi (tmp);
-		oi->ModificationDate=mktime (&tm);
-	}
+	oi->ModificationDate = ptp_unpack_PTPTIME(capture_date);
 	free(capture_date);
 }
 
@@ -558,6 +591,24 @@ ptp_unpack_DPV (
 	case PTP_DTC_UINT32:
 		CTVAL(value->u32,dtoh32a);
 		break;
+	case PTP_DTC_INT64:
+		CTVAL(value->i64,dtoh64a);
+		break;
+	case PTP_DTC_UINT64:
+		CTVAL(value->u64,dtoh64a);
+		break;
+
+	case PTP_DTC_UINT128:
+		*offset += 16;
+		/*fprintf(stderr,"unhandled unpack of uint128n");*/
+		break;
+	case PTP_DTC_INT128:
+		*offset += 16;
+		/*fprintf(stderr,"unhandled unpack of int128n");*/
+		break;
+
+
+
 	case PTP_DTC_AINT8:
 		RARR(value,i8,dtoh8a);
 		break;
@@ -576,6 +627,12 @@ ptp_unpack_DPV (
 	case PTP_DTC_AINT32:
 		RARR(value,i32,dtoh32a);
 		break;
+	case PTP_DTC_AUINT64:
+		RARR(value,u64,dtoh64a);
+		break;
+	case PTP_DTC_AINT64:
+		RARR(value,i64,dtoh64a);
+		break;
 	/* XXX: other int types are unimplemented */
 	/* XXX: other int arrays are unimplemented also */
 	case PTP_DTC_STR: {
@@ -584,9 +641,11 @@ ptp_unpack_DPV (
 		value->str = ptp_unpack_string(params,data,*offset,&len);
 		*offset += len*2+1;
 		if (!value->str)
-			return 0;
+			return 1;
 		break;
 	}
+	default:
+		return 0;
 	}
 	return 1;
 }
@@ -607,10 +666,13 @@ ptp_unpack_DPD (PTPParams *params, unsigned char* data, PTPDevicePropDesc *dpd,
 	dpd->DevicePropertyCode=dtoh16a(&data[PTP_dpd_DevicePropertyCode]);
 	dpd->DataType=dtoh16a(&data[PTP_dpd_DataType]);
 	dpd->GetSet=dtoh8a(&data[PTP_dpd_GetSet]);
+	dpd->FormFlag=PTP_DPFF_None;
 
 	offset = PTP_dpd_FactoryDefaultValue;
 	ret = ptp_unpack_DPV (params, data, &offset, dpdlen, &dpd->FactoryDefaultValue, dpd->DataType);
 	if (!ret) goto outofmemory;
+	if ((dpd->DataType == PTP_DTC_STR) && (offset == dpdlen))
+		return 1;
 	ret = ptp_unpack_DPV (params, data, &offset, dpdlen, &dpd->CurrentValue, dpd->DataType);
 	if (!ret) goto outofmemory;
 
@@ -619,7 +681,6 @@ ptp_unpack_DPD (PTPParams *params, unsigned char* data, PTPDevicePropDesc *dpd,
 	   values). In both cases Form Flag should be set to 0x00 and FORM is
 	   not present. */
 
-	dpd->FormFlag=PTP_DPFF_None;
 	if (offset==PTP_dpd_FactoryDefaultValue)
 		return 1;
 
@@ -777,47 +838,71 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 		dpv=malloc(size);
 		htod32a(dpv,value->u32);
 		break;
+	case PTP_DTC_INT64:
+		size=sizeof(int64_t);
+		dpv=malloc(size);
+		htod64a(dpv,value->i64);
+		break;
+	case PTP_DTC_UINT64:
+		size=sizeof(uint64_t);
+		dpv=malloc(size);
+		htod64a(dpv,value->u64);
+		break;
 	case PTP_DTC_AUINT8:
 		size=sizeof(uint32_t)+value->a.count*sizeof(uint8_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod8a(&dpv[4+i],value->a.v[i].u8);
+			htod8a(&dpv[sizeof(uint32_t)+i*sizeof(uint8_t)],value->a.v[i].u8);
 		break;
 	case PTP_DTC_AINT8:
 		size=sizeof(uint32_t)+value->a.count*sizeof(int8_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod8a(&dpv[4+i],value->a.v[i].i8);
+			htod8a(&dpv[sizeof(uint32_t)+i*sizeof(int8_t)],value->a.v[i].i8);
 		break;
 	case PTP_DTC_AUINT16:
 		size=sizeof(uint32_t)+value->a.count*sizeof(uint16_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod16a(&dpv[4+i],value->a.v[i].u16);
+			htod16a(&dpv[sizeof(uint32_t)+i*sizeof(uint16_t)],value->a.v[i].u16);
 		break;
 	case PTP_DTC_AINT16:
 		size=sizeof(uint32_t)+value->a.count*sizeof(int16_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod16a(&dpv[4+i],value->a.v[i].i16);
+			htod16a(&dpv[sizeof(uint32_t)+i*sizeof(int16_t)],value->a.v[i].i16);
 		break;
 	case PTP_DTC_AUINT32:
 		size=sizeof(uint32_t)+value->a.count*sizeof(uint32_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod32a(&dpv[4+i],value->a.v[i].u32);
+			htod32a(&dpv[sizeof(uint32_t)+i*sizeof(uint32_t)],value->a.v[i].u32);
 		break;
 	case PTP_DTC_AINT32:
 		size=sizeof(uint32_t)+value->a.count*sizeof(int32_t);
 		dpv=malloc(size);
 		htod32a(dpv,value->a.count);
 		for (i=0;i<value->a.count;i++)
-			htod32a(&dpv[4+i],value->a.v[i].i32);
+			htod32a(&dpv[sizeof(uint32_t)+i*sizeof(int32_t)],value->a.v[i].i32);
+		break;
+	case PTP_DTC_AUINT64:
+		size=sizeof(uint32_t)+value->a.count*sizeof(uint64_t);
+		dpv=malloc(size);
+		htod32a(dpv,value->a.count);
+		for (i=0;i<value->a.count;i++)
+			htod64a(&dpv[sizeof(uint32_t)+i*sizeof(uint64_t)],value->a.v[i].u64);
+		break;
+	case PTP_DTC_AINT64:
+		size=sizeof(uint32_t)+value->a.count*sizeof(int64_t);
+		dpv=malloc(size);
+		htod32a(dpv,value->a.count);
+		for (i=0;i<value->a.count;i++)
+			htod64a(&dpv[sizeof(uint32_t)+i*sizeof(int64_t)],value->a.v[i].i64);
 		break;
 	/* XXX: other int types are unimplemented */
 	case PTP_DTC_STR: {
@@ -831,12 +916,13 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 
 #define MAX_MTP_PROPS 127
 static inline uint32_t
-ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldataptr, uint32_t objectid)
+ptp_pack_OPL (PTPParams *params, MTPProperties *props, int nrofprops, unsigned char** opldataptr)
 {
 	unsigned char* opldata;
-	MTPPropList *propitr;
+	MTPProperties *propitr;
 	unsigned char *packedprops[MAX_MTP_PROPS];
 	uint32_t packedpropslens[MAX_MTP_PROPS];
+	uint32_t packedobjecthandles[MAX_MTP_PROPS];
 	uint16_t packedpropsids[MAX_MTP_PROPS];
 	uint16_t packedpropstypes[MAX_MTP_PROPS];
 	uint32_t totalsize = 0;
@@ -845,9 +931,10 @@ ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldatap
 	uint32_t i;
 
 	totalsize = sizeof(uint32_t); /* 4 bytes to store the number of elements */
-	propitr = proplist;
-	while (propitr != NULL && noitems < MAX_MTP_PROPS) {
-		/* Overhead for each item */
+	propitr = props;
+	while (nrofprops-- && noitems < MAX_MTP_PROPS) {
+		/* Object Handle */
+		packedobjecthandles[noitems]=propitr->ObjectHandle;
 		totalsize += sizeof(uint32_t); /* Object ID */
 		/* Metadata type */
 		packedpropsids[noitems]=propitr->property;
@@ -859,7 +946,7 @@ ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldatap
 	        packedpropslens[noitems] = ptp_pack_DPV (params, &propitr->propval, &packedprops[noitems], propitr->datatype);
 		totalsize += packedpropslens[noitems];
 		noitems ++;
-		propitr = propitr->next;
+		propitr ++;
 	}
 
 	/* Allocate memory for the packed property list */
@@ -871,7 +958,7 @@ ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldatap
 	/* Copy into a nice packed list */
 	for (i = 0; i < noitems; i++) {
 		/* Object ID */
-		htod32a(&opldata[bufp],objectid);
+		htod32a(&opldata[bufp],packedobjecthandles[i]);
 		bufp += sizeof(uint32_t);
 		htod16a(&opldata[bufp],packedpropsids[i]);
 		bufp += sizeof(uint16_t);
@@ -886,44 +973,48 @@ ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldatap
 	return totalsize;
 }
 
+static int
+_compare_func(const void* x, const void *y) {
+	const MTPProperties *px = x;
+	const MTPProperties *py = y;
+
+	return px->ObjectHandle - py->ObjectHandle;
+}
+
 static inline int
-ptp_unpack_OPL (PTPParams *params, unsigned char* data, MTPPropList **proplist, unsigned int len)
+ptp_unpack_OPL (PTPParams *params, unsigned char* data, MTPProperties **pprops, unsigned int len)
 { 
 	uint32_t prop_count = dtoh32a(data);
-	MTPPropList *prop = NULL;
+	MTPProperties *props = NULL;
 	int offset = 0, i;
 
 	if (prop_count == 0) {
-		*proplist = NULL;
+		*pprops = NULL;
 		return 0;
 	}
 	data += sizeof(uint32_t);
-	*proplist = malloc(sizeof(MTPPropList));
-	prop = *proplist;
+	props = malloc(prop_count * sizeof(MTPProperties));
+	if (!props) return 0;
 	for (i = 0; i < prop_count; i++) {
-		/* we ignore the object handle */
+		props[i].ObjectHandle = dtoh32a(data);
 		data += sizeof(uint32_t);
 		len -= sizeof(uint32_t);
 
-		prop->property = dtoh32a(data);
+		props[i].property = dtoh16a(data);
 		data += sizeof(uint16_t);
 		len -= sizeof(uint16_t);
 
-		prop->datatype = dtoh32a(data);
+		props[i].datatype = dtoh16a(data);
 		data += sizeof(uint16_t);
 		len -= sizeof(uint16_t);
 
 		offset = 0;
-		ptp_unpack_DPV(params, data, &offset, len, &prop->propval, prop->datatype);
+		ptp_unpack_DPV(params, data, &offset, len, &props[i].propval, props[i].datatype);
 		data += offset;
 		len -= offset;
-
-		if (i != prop_count - 1) {
-			prop->next = malloc(sizeof(MTPPropList));
-			prop = prop->next;
-		} else
-			prop->next = NULL;
 	}
+	qsort (props, prop_count, sizeof(MTPProperties),_compare_func);
+	*pprops = props;
 	return prop_count;
 }
 
@@ -990,6 +1081,212 @@ ptp_unpack_Canon_FE (PTPParams *params, unsigned char* data, PTPCANONFolderEntry
 		fe->Filename[i]=(char)dtoh8a(&data[PTP_cfe_Filename+i]);
 }
 
+/*
+    PTP EOS Changes Entry unpack
+*/
+#define PTP_ece_Size		0
+#define PTP_ece_Type		4
+
+#define PTP_ece_Prop_Subtype	8	/* only for properties */
+#define PTP_ece_Prop_Val_Data	0xc	/* only for properties */
+#define PTP_ece_Prop_Desc_Type	0xc	/* only for property descs */
+#define PTP_ece_Prop_Desc_Count	0x10	/* only for property descs */
+#define PTP_ece_Prop_Desc_Data	0x14	/* only for property descs */
+
+#define PTP_ece_OI_ObjectID	8	/* only for objectinfos */
+#define PTP_ece_OI_OFC		0x0c	/* only for objectinfos */
+#define PTP_ece_OI_Size		0x14	/* only for objectinfos */
+#define PTP_ece_OI_Name		0x1c	/* only for objectinfos */
+
+extern void
+ptp_debug (PTPParams *params, const char *format, ...);
+
+static inline int
+ptp_unpack_CANON_changes (PTPParams *params, unsigned char* data, int datasize, PTPCanon_changes_entry **ce)
+{
+	int	i = 0, entries = 0;
+	unsigned char	*curdata = data;
+
+	if (data==NULL)
+		return 0;
+	while (curdata - data < datasize) {
+		uint32_t	size = dtoh32a(&curdata[PTP_ece_Size]);
+		uint32_t	type = dtoh32a(&curdata[PTP_ece_Type]);
+
+		curdata += size;
+		if ((size == 8) && (type == 0))
+			break;
+		entries++;
+	}
+	*ce = malloc (sizeof(PTPCanon_changes_entry)*(entries+1));
+	if (!*ce) return 0;
+
+	curdata = data;
+	while (curdata - data < datasize) {
+		uint32_t	size = dtoh32a(&curdata[PTP_ece_Size]);
+		uint32_t	type = dtoh32a(&curdata[PTP_ece_Type]);
+
+		switch (type) {
+		case  0xc186: {	/* objectinfo from capture */
+			(*ce)[i].type = PTP_CANON_EOS_CHANGES_TYPE_OBJECTINFO;
+			(*ce)[i].u.object.oid    		= dtoh32a(&curdata[PTP_ece_OI_ObjectID]);
+			(*ce)[i].u.object.oi.ObjectFormat 	= dtoh16a(&curdata[PTP_ece_OI_OFC]);
+			(*ce)[i].u.object.oi.ObjectCompressedSize = dtoh32a(&curdata[PTP_ece_OI_Size]);
+			(*ce)[i].u.object.oi.Filename 		= strdup(((char*)&curdata[PTP_ece_OI_Name]));
+
+			ptp_debug (params, "event %d: objectinfo oid %08lx, ofc %04x, size %d, filename %s", i, (*ce)[i].u.object.oid, (*ce)[i].u.object.oi.ObjectFormat, (*ce)[i].u.object.oi.ObjectCompressedSize, (*ce)[i].u.object.oi.Filename);
+			break;
+		}
+		case  0xc18a: {	/* property desc */
+			uint32_t	proptype = dtoh32a(&curdata[PTP_ece_Prop_Subtype]);
+			uint32_t	propxtype = dtoh32a(&curdata[PTP_ece_Prop_Desc_Type]);
+			uint32_t	propxcnt = dtoh32a(&curdata[PTP_ece_Prop_Desc_Count]);
+			unsigned char	*data = &curdata[PTP_ece_Prop_Desc_Data];
+			int		j;
+			PTPDevicePropDesc	*dpd;
+
+			ptp_debug (params, "event %d: EOS prop %04x desc record, datasize %d", i, proptype, size-PTP_ece_Prop_Desc_Data);
+			for (j=0;j<params->nrofcanon_props;j++)
+				if (params->canon_props[j].proptype == proptype)
+					break;
+			if (j==params->nrofcanon_props) {
+				ptp_debug (params, "event %d: propdesc %x, default value not found.", i, proptype);
+				break;
+			}
+			dpd = &params->canon_props[j].dpd;
+			if (propxtype != 3) {
+				ptp_debug (params, "event %d: propxtype is %x for %04x, unhandled.", i, propxtype, proptype);
+				for (j=0;j<size-PTP_ece_Prop_Desc_Data;j++)
+					ptp_debug (params, "    %d: %02x", j, data[j]);
+				break;
+			}
+			if (propxcnt) {
+				dpd->FormFlag = PTP_DPFF_Enumeration;
+				dpd->FORM.Enum.NumberOfValues = propxcnt;
+				dpd->FORM.Enum.SupportedValue = malloc (sizeof (PTPPropertyValue)*propxcnt);
+				for (j=0;j<propxcnt;j++) {
+					switch (dpd->DataType) {
+					case PTP_DTC_UINT16:
+						dpd->FORM.Enum.SupportedValue[j].u16	= dtoh16a(data);
+						ptp_debug (params, "event %d: suppval[%d] of %x is %x.", i, j, proptype, dtoh16a(data));
+						break;
+					case PTP_DTC_UINT8:
+						dpd->FORM.Enum.SupportedValue[j].u8	= dtoh8a(data);
+						ptp_debug (params,"event %d: suppvalue[%d] of %x is %x", i, j, proptype, dtoh8a(data));
+						break;
+					default: {
+						int k;
+						ptp_debug (params ,"event %d: data type 0x%04x of %x unhandled, fill in (val=%x).", i, dpd->DataType, proptype, dtoh32a(data));
+						for (k=0;k<size-PTP_ece_Prop_Desc_Data;k++)
+							ptp_debug (params, "    %d: %02x", k, data[k]);
+						break;
+					}
+					}
+					data += 4; /* might only be for propxtype 3 */
+				}
+			}
+			break;
+		}
+		case  0xc189:	/* property value */
+			if (size >= 0xc) {	/* property info */
+				int j;
+				uint32_t	proptype = dtoh32a(&curdata[PTP_ece_Prop_Subtype]);
+				unsigned char	*data = &curdata[PTP_ece_Prop_Val_Data];
+				PTPDevicePropDesc	*dpd;
+
+				ptp_debug (params, "event %d: EOS prop %04x info record, datasize %d", i, proptype, size-4);
+				for (j=0;j<params->nrofcanon_props;j++)
+					if (params->canon_props[j].proptype == proptype)
+						break;
+				if (j<params->nrofcanon_props) {
+					if (	(params->canon_props[j].size != size) ||
+						(memcmp(params->canon_props[j].data,data,size-PTP_ece_Prop_Val_Data))) {
+						params->canon_props[j].data = realloc(params->canon_props[j].data,size-PTP_ece_Prop_Val_Data);
+						memcpy (params->canon_props[j].data,data,size-PTP_ece_Prop_Val_Data);
+					}
+				} else {
+					if (j)
+						params->canon_props = realloc(params->canon_props, sizeof(params->canon_props[0])*(j+1));
+					else
+						params->canon_props = malloc(sizeof(params->canon_props[0]));
+					params->canon_props[j].type = type;
+					params->canon_props[j].proptype = proptype;
+					params->canon_props[j].size = size;
+					params->canon_props[j].data = malloc(size-PTP_ece_Prop_Val_Data);
+					memcpy(params->canon_props[j].data, data, size-PTP_ece_Prop_Val_Data);
+					memset (&params->canon_props[j].dpd,0,sizeof(params->canon_props[j].dpd));
+					params->canon_props[j].dpd.GetSet = 1;
+					params->canon_props[j].dpd.FormFlag = PTP_DPFF_None;
+					params->nrofcanon_props = j+1;
+				}
+				dpd = &params->canon_props[j].dpd;
+				switch (proptype) {
+				case PTP_DPC_CANON_EOS_CameraTime:
+					dpd->DataType = PTP_DTC_UINT32;
+					break;
+				case PTP_DPC_CANON_EOS_Aperture:
+				case PTP_DPC_CANON_EOS_ShutterSpeed:
+				case PTP_DPC_CANON_EOS_ISOSpeed:
+					dpd->DataType = PTP_DTC_UINT16;
+					break;
+				case PTP_DPC_CANON_EOS_PictureStyle:
+				case PTP_DPC_CANON_EOS_WhiteBalance:
+				case PTP_DPC_CANON_EOS_MeteringMode:
+				case PTP_DPC_CANON_EOS_ExpCompensation:
+					dpd->DataType = PTP_DTC_UINT8;
+					break;
+				case PTP_DPC_CANON_EOS_Owner:
+					dpd->DataType = PTP_DTC_STR;
+					break;
+				default:
+					ptp_debug (params, "event %d: Unknown EOS property %04x, datasize is %d", i ,proptype, size-PTP_ece_Prop_Val_Data);
+					for (j=0;j<size-PTP_ece_Prop_Val_Data;j++)
+						ptp_debug (params, "    %d: %02x", j, data[j]);
+					break;
+				}
+				switch (dpd->DataType) {
+				case PTP_DTC_UINT32:
+					dpd->FactoryDefaultValue.u32	= dtoh32a(data);
+					dpd->CurrentValue.u32		= dtoh32a(data);
+					ptp_debug (params ,"event %d: currentvalue of %x is %x", i, proptype, dpd->CurrentValue.u32);
+					break;
+				case PTP_DTC_UINT16:
+					dpd->FactoryDefaultValue.u16	= dtoh16a(data);
+					dpd->CurrentValue.u16		= dtoh16a(data);
+					ptp_debug (params,"event %d: currentvalue of %x is %x", i, proptype, dpd->CurrentValue.u16);
+					break;
+				case PTP_DTC_UINT8:
+					dpd->FactoryDefaultValue.u8	= dtoh8a(data);
+					dpd->CurrentValue.u8		= dtoh8a(data);
+					ptp_debug (params,"event %d: currentvalue of %x is %x", i, proptype, dpd->CurrentValue.u8);
+					break;
+				case PTP_DTC_STR: {
+					uint8_t len = 0;
+					dpd->FactoryDefaultValue.str	= ptp_unpack_string(params, data, 0, &len);
+					dpd->CurrentValue.str		= ptp_unpack_string(params, data, 0, &len);
+					ptp_debug (params,"event %d: currentvalue of %x is %s", i, proptype, dpd->CurrentValue.u8);
+					break;
+				}
+				default:
+					/* debug is printed in switch above this one */
+					break;
+				}
+				break;
+		}
+		default:
+			ptp_debug (params, "event %d: unknown EOS property type %04x", i, type);
+			(*ce)[i].type = PTP_CANON_EOS_CHANGES_TYPE_UNKNOWN;
+			break;
+		}
+		curdata += size;
+		i++;
+		if ((size == 8) && (type == 0))
+			break;
+	}
+
+	return entries;
+}
+
 /*
     PTP USB Event container unpack for Nikon events.
 */
