@@ -19,6 +19,7 @@
  */
 
 /* ptp2 camlib private functions */
+#include <gphoto2/gphoto2-library.h>
 
 /* config.c */
 int camera_get_config (Camera *camera, CameraWidget **window, GPContext *context);
@@ -29,8 +30,15 @@ int camera_unprepare_capture (Camera *camera, GPContext *context);
 /* library.c */
 void report_result (GPContext *context, short result, short vendor);
 int translate_ptp_result (short result);
+void fixup_cached_deviceinfo (Camera *camera);
 
 struct _CameraPrivateLibrary {
 	PTPParams params;
 	unsigned long bugs;
 };
+
+struct _PTPData {
+	Camera *camera;
+	GPContext *context;
+};
+typedef struct _PTPData PTPData;
