@@ -30,6 +30,9 @@
 #include "config.h"
 #include <gphoto2/gphoto2-port-library.h>
 
+/* Solaris needs this */
+#define __EXTENSIONS__
+
 #include <stdlib.h>
 #include <stdio.h>
 #ifdef HAVE_UNISTD_H
@@ -104,6 +107,11 @@
 #define GP_PORT_SERIAL_PREFIX   "/dev/ttyS%i"
 #define GP_PORT_SERIAL_RANGE_LOW        0
 #define GP_PORT_SERIAL_RANGE_HIGH       32
+
+#ifndef IXANY
+#define IXANY  0004000
+#endif
+
 #endif
 
 /* FreeBSD */
@@ -352,7 +360,7 @@ gp_port_library_list (GPPortInfoList *list)
 		/* fall through to standard open if this failed */
 #endif
 		if (fd == -1)
-			fd = open (path, O_RDONLY | O_NDELAY);
+			fd = open (path, O_RDONLY | O_NONBLOCK);
 		if (fd < 0) {
 			gp_port_serial_unlock (NULL, path);
 			continue;
@@ -486,6 +494,7 @@ gp_port_serial_close (GPPort *dev)
 
 	/* Unlock the port */
 	path = strchr (dev->settings.serial.port, ':');
+	if (!path) return GP_ERROR_BAD_PARAMETERS;
 	path++;
 	CHECK (gp_port_serial_unlock (dev, path));
 
@@ -583,10 +592,15 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
 			return (GP_ERROR_TIMEOUT);
 
 		if (dev->settings.serial.parity != GP_PORT_SERIAL_PARITY_OFF) {
+		    unsigned char ffchar[1];
+		    unsigned char nullchar[1];
+
+		    ffchar[0] = 0xff;
+		    nullchar[0] = 0;
 		    now = read (dev->pl->fd, bytes, 1);
 		    if (now < 0)
 			    return GP_ERROR_IO_READ;
-		    if (bytes[0] == (char)-1) {
+		    if (!memcmp(bytes,ffchar,1)) {
 			now = read (dev->pl->fd, bytes, 1);
 			if (now < 0)
 				return GP_ERROR_IO_READ;
@@ -601,12 +615,12 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
 			 *
 			 * cf. man tcsetattr, description of PARMRK.
 			 */
-			if (bytes[0] == 0x00) {
+			if (!memcmp(bytes,nullchar,1)) {
 			    gp_port_set_error (dev, _("Parity error."));
 			    return GP_ERROR_IO_READ;
 			}
-			if (bytes[0] != (char)-1) {
-			    gp_port_set_error (dev, _("Unexpected parity response sequence 0xff 0x%02x."), bytes[0]);
+			if (!memcmp(bytes,ffchar,1)) {
+			    gp_port_set_error (dev, _("Unexpected parity response sequence 0xff 0x%02x."), ((unsigned char*)bytes)[0]);
 			    return GP_ERROR_IO_READ;
 			}
 			/* Ok, we read 1 byte and it is 0xff */
