@@ -53,6 +53,11 @@ ARGV0 " prints the camera list in the specified format FORMAT on stdout.\n" \
 #include <gphoto2/gphoto2-camera.h>
 #include <gphoto2/gphoto2-port-log.h>
 
+/* for detailed version message */
+#include <gphoto2/gphoto2-version.h>
+
+#include "config.h"
+
 #ifndef TRUE
 #define TRUE  (0==0)
 #endif
@@ -60,6 +65,19 @@ ARGV0 " prints the camera list in the specified format FORMAT on stdout.\n" \
 #define FALSE (0!=0)
 #endif
 
+typedef struct {
+	char *name;
+	GPVersionFunc version_func;
+} module_version;
+
+
+
+const module_version module_versions[] = {
+	{ "libgphoto2", gp_library_version },
+	{ "libgphoto2_port", gp_port_library_version },
+	{ NULL, NULL }
+};
+
 typedef char *string_array_t[];
 
 typedef string_array_t *string_array_p;
@@ -72,12 +90,15 @@ typedef struct {
 } func_params_t;
 
 
-typedef int (* begin_func_t)  (const func_params_t *params);
+typedef int (* begin_func_t)  (const func_params_t *params,
+			       void **data);
 typedef int (* camera_func_t) (const func_params_t *params, 
 			       const int i,
 			       const int total,
-			       const CameraAbilities *ca);
-typedef int (* end_func_t)    (const func_params_t *params);
+			       const CameraAbilities *ca,
+			       void *data);
+typedef int (* end_func_t)    (const func_params_t *params,
+			       void *data);
 
 
 #define GP_USB_HOTPLUG_MATCH_VENDOR_ID          0x0001
@@ -118,6 +139,68 @@ typedef int (* end_func_t)    (const func_params_t *params);
 		exit(13); \
 	} while (0)
 
+#define ASSERT(cond) \
+	do { \
+		if (!(cond)) { \
+			FATAL("Assertion failed: %s", #cond); \
+		} \
+	} while (0)
+
+
+/* print_version_comment
+ * Print comment to output containing information on library versions
+ *
+ * out        the file to write the comment to
+ * startline  printed at the start of each line, e.g. "# " or "    | "
+ * endline    printed as the end of each line,   e.g. "\n" or "\n"
+ * firstline  printed before first line,         e.g. NULL or "<!--+\n"
+ * lastline   printed after last line,           e.g. "\n" or "    +-->\n"
+ */
+
+static void
+print_version_comment(FILE *out,
+		      const char *startline, const char *endline,
+		      const char *firstline, const char *lastline)
+{
+	unsigned int n;
+	if (out == NULL) { FATAL("Internal error: NULL out in print_version_comment()"); }
+	if (firstline != NULL) { fputs(firstline, out); }
+	fputs(startline, out);
+	fputs("Created from this library:", out);
+	fputs(endline, out);
+	for (n=0; (module_versions[n].name != NULL) && (module_versions[n].version_func != NULL); n++) {
+		const char *name = module_versions[n].name;
+		GPVersionFunc func = module_versions[n].version_func;
+		const char **v = func(GP_VERSION_SHORT);
+		unsigned int i;
+		if (!v) { continue; }
+		if (!v[0]) { continue; }
+		if (startline != NULL) { fputs(startline, out); }
+		fputs("  ", out);
+		fprintf(out,"%-15s %-14s ", name, v[0]);
+		for (i=1; v[i] != NULL; i++) {
+			fputs(v[i], out);
+			if (v[i+1] != NULL) {
+				fputs(", ", out);
+			}
+		}
+		if (endline != NULL) { fputs(endline, out); }
+	}
+	if (lastline != NULL) { fputs(lastline, out); }
+}
+
+
+static int
+hotplug_begin_func (const func_params_t *params, void **data)
+{
+	if (params->add_comments) {
+		printf("# linux-hotplug configuration file "
+		       "for libgphoto2 supported devices\n");
+		print_version_comment(stdout, "# ", "\n", NULL, "#\n");
+	}
+	return 0;
+}
+
 
 /* print_usb_usermap
  *
@@ -132,7 +215,8 @@ static int
 hotplug_camera_func (const func_params_t *params, 
 		     const int i,
 		     const int total,
-		     const CameraAbilities *a)
+		     const CameraAbilities *a,
+		     void *data)
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
@@ -213,7 +297,7 @@ print_hline (void)
 
 
 static int
-human_begin_func (const func_params_t *params)
+human_begin_func (const func_params_t *params, void **data)
 {
 	print_hline();
 	print_headline();
@@ -223,7 +307,7 @@ human_begin_func (const func_params_t *params)
 
 
 static int
-human_end_func (const func_params_t *params)
+human_end_func (const func_params_t *params, void *data)
 {
 	print_hline();
 	print_headline();
@@ -251,7 +335,8 @@ static int
 human_camera_func (const func_params_t *params, 
 		   const int i, 
 		   const int total,
-		   const CameraAbilities *a)
+		   const CameraAbilities *a,
+		   void *data)
 {
 	const char *camlib_basename;
 	camlib_basename = basename(a->library);
@@ -266,9 +351,10 @@ human_camera_func (const func_params_t *params,
 
 static int
 idlist_camera_func (const func_params_t *params, 
-		   const int i, 
-		   const int total,
-		   const CameraAbilities *a)
+		    const int i, 
+		    const int total,
+		    const CameraAbilities *a,
+		    void *data)
 {
 	if (a->usb_vendor) { /* usb product id may be zero! */
 		printf("%04x:%04x %s\n",
@@ -280,33 +366,131 @@ idlist_camera_func (const func_params_t *params,
 }
 
 
-static int
-udev_begin_func (const func_params_t *params)
+typedef enum {
+		UDEV_PRE_0_98 = 0,
+		UDEV_0_98 = 1
+} udev_version_t;
+
+static const StringFlagItem udev_version_t_map[] = {
+	{ "pre-0.98", UDEV_PRE_0_98 },
+	{ "0.98", UDEV_0_98 },
+	{ NULL, 0 }
+};
+
+
+typedef struct {
+	udev_version_t version;
+	char *mode;
+	char *owner;
+	char *group;
+	char *script;
+	const char *begin_string;
+	const char *usbcam_string;
+} udev_persistent_data_t;
+
+
+static void
+udev_parse_params (const func_params_t *params, void **data)
 {
-	printf ("# udev rules file for libgphoto2 devices (udev < 0.98)\n#\n");
 	/* Note: 2 lines because we need to use || ... having them on the same
 	 * line would mean &&.
 	 */
-	printf ("BUS!=\"usb*\", GOTO=\"libgphoto2_rules_end\"\n");
-	printf ("ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n");
+	static const char * const begin_strings[] = {
+		/* UDEV_PRE_0_98 */
+		"BUS!=\"usb_device\", GOTO=\"libgphoto2_rules_end\"\n"
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n",
+		/* UDEV_0_98 */
+		"SUBSYSTEM!=\"usb|usb_device\", GOTO=\"libgphoto2_rules_end\"\n"
+		"ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n"
+	};
+	static const char * const usbcam_strings[] = {
+		/* UDEV_PRE_0_98 */
+		"SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\", ",
+		/* UDEV_0_98 */
+		"ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ",
+	};
+	udev_persistent_data_t *pdata;
+	pdata = calloc(1, sizeof(udev_persistent_data_t));
+	pdata->version = UDEV_0_98;
+	ASSERT(data != NULL);
+	*data = (void *) pdata;
+	if (1) {
+		int i;
+		char *key = NULL, *val = NULL;
+		for (i=0; ((key=(*params->argv)[i])   != NULL) && 
+			  ((val=(*params->argv)[i+1]) != NULL); i+=2) {
+			if (0) {
+				/* nothing */
+			} else if (strcmp("script", key)==0) {
+				pdata->script = val;
+			} else if (strcmp("owner", key)==0) {
+				pdata->owner = val;
+			} else if (strcmp("group", key)==0) {
+				pdata->group = val;
+			} else if (strcmp("version", key)==0) {
+				if (gpi_string_to_enum(val, &(pdata->version),
+						       udev_version_t_map)) {
+					FATAL("Unrecognized udev version: \"%s\"", val);
+				}
+			} else if (strcmp("mode", key)==0) {
+				pdata->mode = val;
+			} else {
+				FATAL("Unknown key argument: %s", key);
+			}
+		}
+		if ((key != NULL) && (val == NULL)) {
+			FATAL("Single argument remaining; need pairs of key and value");
+		}
+	}
+	if ((0==0)
+	    && pdata->mode == NULL 
+	    && pdata->group == NULL 
+	    && pdata->owner == NULL 
+	    && pdata->script == NULL) {
+		FATAL("Either <script> or <mode,group,owner> parameters must be given.");
+	}
+	if ((pdata->script != NULL) && (pdata->mode != NULL 
+					|| pdata->group != NULL 
+					|| pdata->owner != NULL)) {
+		FATAL("The <script> parameter conflicts with the <mode,group,owner> parameters.");
+	}
+
+	pdata->begin_string = begin_strings[pdata->version];
+	pdata->usbcam_string = usbcam_strings[pdata->version];
+}
+
+
+static const char *
+get_version_str(udev_version_t version)
+{
+	return gpi_enum_to_string(version, udev_version_t_map);
+}
+
+
+static int
+udev_begin_func (const func_params_t *params, void **data)
+{
+	udev_parse_params(params, data);
+	if (1) {
+		udev_persistent_data_t *pdata = (udev_persistent_data_t *) (*data);
+		const char *version_str = get_version_str(pdata->version);
+		ASSERT(pdata != NULL);
+		ASSERT(version_str != NULL);
+		printf ("# udev rules file for libgphoto2 devices (for udev %s version)\n",
+			version_str);
+		print_version_comment(stdout, "# ", "\n", NULL, "#\n");
+		printf (pdata->begin_string);
+	}
 	return 0;
 }
 
-static int
-udev_098_begin_func (const func_params_t *params)
-{
-	printf ("# udev rules file for libgphoto2 devices (udev >= 0.98)\n#\n");
-	/* Note: 2 lines because we need to use || ... having them on the same
-	 * line would mean &&.
-	 */
-	printf ("SUBSYSTEM!=\"usb*\", GOTO=\"libgphoto2_rules_end\"\n");
-	printf ("ACTION!=\"add\", GOTO=\"libgphoto2_rules_end\"\n\n");
-	return 0;
-}
 
 static int
-udev_end_func (const func_params_t *params)
+udev_end_func (const func_params_t *params, void *data)
 {
+	if (data != NULL) {
+		free(data);
+	}
 	printf ("\nLABEL=\"libgphoto2_rules_end\"\n");
 	return 0;
 }
@@ -318,22 +502,29 @@ static int
 udev_camera_func (const func_params_t *params, 
 		  const int i,
 		  const int total,
-		  const CameraAbilities *a)
+		  const CameraAbilities *a,
+		  void *data)
 {
 	int flags = 0;
 	int class = 0, subclass = 0, proto = 0;
 	int usb_vendor = 0, usb_product = 0;
+	int has_valid_rule = 0;
+	udev_persistent_data_t *pdata = (udev_persistent_data_t *) data;
+	ASSERT(pdata != NULL);
 
-	if (a->port & GP_PORT_USB) {
-		if (a->usb_vendor) { /* usb product id may be zero! */
-			class = 0;
-			subclass = 0;
-			proto = 0;
-			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
-				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
-			usb_vendor = a->usb_vendor;
-			usb_product = a->usb_product;
-		} else if ((a->usb_class) && (a->usb_class != 666)) {
+	if (!(a->port & GP_PORT_USB))
+		return 0;
+
+	if (a->usb_vendor) { /* usb product id may be zero! */
+		class = 0;
+		subclass = 0;
+		proto = 0;
+		flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
+			 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
+		usb_vendor = a->usb_vendor;
+		usb_product = a->usb_product;
+	} else {
+		if (a->usb_class) {
 			class = a->usb_class;
 			subclass = a->usb_subclass;
 			proto = a->usb_protocol;
@@ -349,9 +540,6 @@ udev_camera_func (const func_params_t *params,
 			usb_vendor = 0;
 			usb_product = 0;
 		}
-	} else {
-		/* not a camera handled by udev */
-		return 0;
 	}
 
 	if (params->add_comments) {
@@ -361,209 +549,81 @@ udev_camera_func (const func_params_t *params,
 	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
 		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) {
 			printf("PROGRAM=\"check-ptp-camera %02d/%02d/%02d\", ", class, subclass, proto);
+			has_valid_rule = 1;
 		} else {
 			if (class == 666) {
 				printf("# not working yet: PROGRAM=\"check-mtp-device\", ");
+				has_valid_rule = 1;
 			} else {
-				fprintf(stderr,"unhandled interface match flags %x\n", flags);
+				fprintf(stderr, "unhandled interface match flags %x\n", flags);
 			}
 		}
 	} else {
-		printf ("SYSFS{idVendor}==\"%04x\", SYSFS{idProduct}==\"%04x\", ",
-			a->usb_vendor, a->usb_product);
+		if (flags & GP_USB_HOTPLUG_MATCH_VENDOR_ID) {
+			printf (pdata->usbcam_string, a->usb_vendor, a->usb_product);
+			has_valid_rule = 1;
+		} else {
+			fprintf (stderr, "Error: Trying to output device %d/%d with incorrect match flags.\n",
+				a->usb_vendor, a->usb_product
+			);
+		}
 	}
-	if ((*params->argv)[1] == NULL) {
-		const char *hotplug_script = ((*params->argv)[0] != NULL)
-			?((*params->argv)[0])
-			:("/etc/hotplug/usb/" GP_USB_HOTPLUG_SCRIPT);
-		printf("RUN+=\"%s\"\n", hotplug_script);
-	} else { /* more than two parameters in params->argv */
-		char *mode = NULL;
-		char *owner = NULL;
-		char *group = NULL;
-
-		int i;
-		char *key = NULL, *val = NULL;
-		for (i=0; ((key=(*params->argv)[i])   != NULL) && 
-			  ((val=(*params->argv)[i+1]) != NULL); i+=2) {
-			if (strcmp("owner", key)==0) {
-				owner = val;
-			} else if (strcmp("group", key)==0) {
-				group = val;
-			} else if (strcmp("mode", key)==0) {
-				mode = val;
-			} else {
-				FATAL("Unknown key argument: %s", key);
-			}
-		}
-		if ((key != NULL) && (val == NULL)) {
-			FATAL("Single argument remaining; need pairs of key and value");
-		}
-
-		if (mode != NULL || owner != NULL || group != NULL) {
-			if (mode != NULL) {
-				printf("MODE=\"%s\"", mode);
-				if (owner != NULL || group != NULL) {
+	if (has_valid_rule != 0) {
+		if (pdata->script != NULL) {
+			printf("RUN+=\"%s\"\n", pdata->script);
+		} else if (pdata->mode != NULL || pdata->owner != NULL || pdata->group != NULL) {
+			if (pdata->mode != NULL) {
+				printf("MODE=\"%s\"", pdata->mode);
+				if (pdata->owner != NULL || pdata->group != NULL) {
 					printf(", ");
 				}
 			}
-			if (owner != NULL) {
-				printf("OWNER=\"%s\"", owner);
-				if (group != NULL) {
+			if (pdata->owner != NULL) {
+				printf("OWNER=\"%s\"", pdata->owner);
+				if (pdata->group != NULL) {
 					printf(", ");
 				}
 			}
-			if (group != NULL) {
-				printf("GROUP=\"%s\"", group);
+			if (pdata->group != NULL) {
+				printf("GROUP=\"%s\"", pdata->group);
 			}
 			printf("\n");
 		} else {
-			FATAL("Arguments required");
-		}
-	}
-	/*
-	printf("MUH!\n");
-	*/
-	return 0;
-}
-
-static int
-udev_098_camera_func (const func_params_t *params, 
-		  const int i,
-		  const int total,
-		  const CameraAbilities *a)
-{
-	int flags = 0;
-	int class = 0, subclass = 0, proto = 0;
-	int usb_vendor = 0, usb_product = 0;
-
-	if (a->port & GP_PORT_USB) {
-		if (a->usb_vendor) { /* usb product id may be zero! */
-			class = 0;
-			subclass = 0;
-			proto = 0;
-			flags = (GP_USB_HOTPLUG_MATCH_VENDOR_ID 
-				 | GP_USB_HOTPLUG_MATCH_PRODUCT_ID);
-			usb_vendor = a->usb_vendor;
-			usb_product = a->usb_product;
-		} else if ((a->usb_class) && (a->usb_class != 666)) {
-			class = a->usb_class;
-			subclass = a->usb_subclass;
-			proto = a->usb_protocol;
-			flags = GP_USB_HOTPLUG_MATCH_INT_CLASS;
-			if (subclass != -1)
-				flags |= GP_USB_HOTPLUG_MATCH_INT_SUBCLASS;
-			else
-				subclass = 0;
-			if (proto != -1)
-				flags |= GP_USB_HOTPLUG_MATCH_INT_PROTOCOL;
-			else
-				proto = 0;
-			usb_vendor = 0;
-			usb_product = 0;
-		}
-	} else {
-		/* not a camera handled by udev */
-		return 0;
-	}
-
-	if (params->add_comments)
-		printf ("# %s\n", a->model);
-
-	if (flags & GP_USB_HOTPLUG_MATCH_INT_CLASS) {
-		if ((flags & (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) == (GP_USB_HOTPLUG_MATCH_INT_CLASS|GP_USB_HOTPLUG_MATCH_INT_SUBCLASS|GP_USB_HOTPLUG_MATCH_INT_PROTOCOL)) {
-			printf("PROGRAM=\"check-ptp-camera %02d/%02d/%02d\", ", class, subclass, proto);
-		} else {
-			if (class == 666)
-				printf("# not working yet: PROGRAM=\"check-mtp-device\", ");
-			else
-				fprintf(stderr,"unhandled interface match flags %x\n", flags);
-		}
-	} else {
-		printf ("ATTRS{idVendor}==\"%04x\", ATTRS{idProduct}==\"%04x\", ",
-			a->usb_vendor, a->usb_product);
-	}
-	if ((*params->argv)[1] == NULL) {
-		const char *hotplug_script = ((*params->argv)[0] != NULL)
-			?((*params->argv)[0])
-			:("/etc/hotplug/usb/" GP_USB_HOTPLUG_SCRIPT);
-		printf("RUN+=\"%s\"\n", hotplug_script);
-	} else { /* more than two parameters in params->argv */
-		char *mode = NULL;
-		char *owner = NULL;
-		char *group = NULL;
-
-		int i;
-		char *key = NULL, *val = NULL;
-		for (i=0; ((key=(*params->argv)[i])   != NULL) && 
-			  ((val=(*params->argv)[i+1]) != NULL); i+=2) {
-			if (strcmp("owner", key)==0) {
-				owner = val;
-			} else if (strcmp("group", key)==0) {
-				group = val;
-			} else if (strcmp("mode", key)==0) {
-				mode = val;
-			} else {
-				FATAL("Unknown key argument: %s", key);
-			}
-		}
-		if ((key != NULL) && (val == NULL)) {
-			FATAL("Single argument remaining; need pairs of key and value");
-		}
-
-		if (mode != NULL || owner != NULL || group != NULL) {
-			if (mode != NULL) {
-				printf("MODE=\"%s\"", mode);
-				if (owner != NULL || group != NULL) {
-					printf(", ");
-				}
-			}
-			if (owner != NULL) {
-				printf("OWNER=\"%s\"", owner);
-				if (group != NULL) {
-					printf(", ");
-				}
-			}
-			if (group != NULL) {
-				printf("GROUP=\"%s\"", group);
-			}
 			printf("\n");
-		} else {
-			FATAL("Arguments required");
+			FATAL("udev_camera_func(): illegal branch");
 		}
 	}
-	/*
-	printf("MUH!\n");
-	*/
 	return 0;
 }
 
 
-
 static int
-empty_begin_func (const func_params_t *params)
+empty_begin_func (const func_params_t *params, void **data)
 {
 	return 0;
 }
 
 static int
-empty_end_func (const func_params_t *params)
+empty_end_func (const func_params_t *params, void *data)
 {
 	return 0;
 }
 
 
 
+#ifdef ENABLED_GP2DDB
+
 static int
-ddb_begin_func (const func_params_t *params)
+ddb_begin_func (const func_params_t *params, void **data)
 {
 	printf("# Beginning of gphoto2 device database (PRE-ALPHA format!)\n\n");
+	print_version_comment(stdout, "# ", "\n", NULL, "\n");
 	return 0;
 }
 
 
 static int
-ddb_end_func (const func_params_t *params)
+ddb_end_func (const func_params_t *params, void *data)
 {
 	printf("# End of gphoto2 device database (PRE-ALPHA format!).\n");
 	return 0;
@@ -580,12 +640,11 @@ ddb_list_out_func(const char *str, void *data)
 
 
 static void
-ddb_delayed_head(
-	const func_params_t *params, 
-	const int i,
-	const int total,
-	const CameraAbilities *a,
-	const char *camlib_basename)
+ddb_delayed_head(const func_params_t *params, 
+		 const int i,
+		 const int total,
+		 const CameraAbilities *a,
+		 const char *camlib_basename)
 {
 	int first = 1;
 	printf("# This is a PRE-ALPHA data format. Do not use it.\n\n");
@@ -645,7 +704,8 @@ static int
 ddb_camera_func (const func_params_t *params, 
 		 const int i,
 		 const int total,
-		 const CameraAbilities *a)
+		 const CameraAbilities *a,
+		 void *data)
 {
 	const char *camlib_basename = basename(a->library);
 	int head_printed = 0;
@@ -719,6 +779,8 @@ ddb_camera_func (const func_params_t *params,
 	return 0;
 }
 
+#endif /* ENABLED_GP2DDB */
+
 
 /* print_fdi_map
  *
@@ -730,14 +792,15 @@ ddb_camera_func (const func_params_t *params,
  */
 
 static int
-fdi_begin_func (const func_params_t *params)
+fdi_begin_func (const func_params_t *params, void **data)
 {
 	printf("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?> <!-- -*- SGML -*- -->\n");
 	printf("<!-- This file was generated by %s - - fdi -->\n",
 	       "libgphoto2 " ARGV0);
+	print_version_comment(stdout, "    | ", "\n", "<!--+\n", "    +-->\n");
 	printf("<deviceinfo version=\"0.2\">\n");
 	printf(" <device>\n");
-	printf("  <match key=\"info.bus\" string=\"usb\">\n");
+	printf("  <match key=\"info.subsystem\" string=\"usb\">\n");
 	return 0;
 }
 
@@ -746,7 +809,8 @@ static int
 fdi_camera_func (const func_params_t *params, 
 		 const int i,
 		 const int total,
-		 const CameraAbilities *a)
+		 const CameraAbilities *a,
+		 void *data)
 {
 	char	*s, *d, model[256];
 
@@ -755,7 +819,7 @@ fdi_camera_func (const func_params_t *params,
 
 	s = (char *) a->model;
 	d = model;
-	while (*s) {
+	while (*s && (d < &d[sizeof(d)-1])) {
 		if (*s == '&') {
 			strcpy(d,"&amp;");
 			d += strlen(d);
@@ -767,7 +831,7 @@ fdi_camera_func (const func_params_t *params,
 	*d = '\0';
 
 	if ((a->port & GP_PORT_USB)) {
-		if (a->usb_vendor == 0x07b4 && a->usb_product == 0x105) {
+		if (a->usb_vendor == 0x07b4 && (a->usb_product == 0x105 || a->usb_product == 0x109) ) {
 			/* Marcus says: The Olympus Sierra/Storage dual mode camera firmware.
 			 * Some HAL using software gets deeply confused by this being here
 			 * and also detected as mass storage elsewhere, so blacklist
@@ -778,9 +842,7 @@ fdi_camera_func (const func_params_t *params,
 		if (a->usb_vendor) { /* usb product id might be 0! */
 			printf("   <match key=\"usb.vendor_id\" int=\"%d\">\n", a->usb_vendor);
 			printf("    <match key=\"usb.product_id\" int=\"%d\">\n", a->usb_product);
-			printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
 			if (a->device_type & GP_DEVICE_AUDIO_PLAYER) {
-				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
 				printf("     <merge key=\"info.category\" type=\"string\">portable_audio_player</merge>\n");
 				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
 				printf("     <merge key=\"portable_audio_player.access_method\" type=\"string\">user</merge>\n");
@@ -789,6 +851,7 @@ fdi_camera_func (const func_params_t *params,
 				/* FIXME: needs true formats ... But all of them can do MP3 */
 				printf("     <append key=\"portable_audio_player.output_formats\" type=\"strlist\">audio/mpeg</append>\n");
 			} else {
+				printf("     <merge key=\"info.category\" type=\"string\">camera</merge>\n");
 				printf("     <append key=\"info.capabilities\" type=\"strlist\">camera</append>\n");
 				
 				/* HACK alert ... but the HAL / gnome-volume-manager guys want that */
@@ -829,7 +892,7 @@ fdi_camera_func (const func_params_t *params,
 }
 
 static int
-fdi_end_func (const func_params_t *params)
+fdi_end_func (const func_params_t *params, void *data)
 {
 	printf("  </match>\n");
 	printf(" </device>\n");
@@ -848,14 +911,15 @@ fdi_end_func (const func_params_t *params)
  */
 
 static int
-fdi_device_begin_func (const func_params_t *params)
+fdi_device_begin_func (const func_params_t *params, void **data)
 {
 	printf("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?> <!-- -*- SGML -*- -->\n");
 	printf("<!-- This file was generated by %s - - fdi-device -->\n",
 	       "libgphoto2 " ARGV0);
+	print_version_comment(stdout, "    | ", "\n", "<!--+\n", "    +-->\n");
 	printf("<deviceinfo version=\"0.2\">\n");
 	printf(" <device>\n");
-	printf("  <match key=\"info.bus\" string=\"usb\">\n");
+	printf("  <match key=\"info.subsystem\" string=\"usb\">\n");
 	return 0;
 }
 
@@ -864,7 +928,8 @@ static int
 fdi_device_camera_func (const func_params_t *params, 
 		 	const int i,
 			const int total,
-		 	const CameraAbilities *a)
+		 	const CameraAbilities *a,
+			void *data)
 {
 	char	*s, *d, model[256];
 
@@ -873,7 +938,7 @@ fdi_device_camera_func (const func_params_t *params,
 
 	s = (char *) a->model;
 	d = model;
-	while (*s) {
+	while (*s && (d < &d[sizeof(d)-1])) {
 		if (*s == '&') {
 			strcpy(d,"&amp;");
 			d += strlen(d);
@@ -886,7 +951,7 @@ fdi_device_camera_func (const func_params_t *params,
 
 	if ((a->port & GP_PORT_USB)) {
 
-		if (a->usb_vendor == 0x07b4 && a->usb_product == 0x105) {
+		if (a->usb_vendor == 0x07b4 && (a->usb_product == 0x105 || a->usb_product == 0x109)) {
 			/* Marcus says: The Olympus Sierra/Storage dual mode camera firmware.
 			 * Some HAL using software gets deeply confused by this being here
 			 * and also detected as mass storage elsewhere, so blacklist
@@ -900,6 +965,9 @@ fdi_device_camera_func (const func_params_t *params,
 			 */
 			printf("   <match key=\"usb_device.vendor_id\" int=\"%d\">\n", a->usb_vendor);
 			printf("    <match key=\"usb_device.product_id\" int=\"%d\">\n", a->usb_product);
+			if (params->add_comments) {
+				printf("     <!-- %s -->\n", a->model);
+			}
 			if (a->device_type & GP_DEVICE_AUDIO_PLAYER)
 				printf("     <append key=\"info.capabilities\" type=\"strlist\">portable_audio_player</append>\n");
 			else
@@ -924,7 +992,7 @@ fdi_device_camera_func (const func_params_t *params,
 }
 
 static int
-fdi_device_end_func (const func_params_t *params)
+fdi_device_end_func (const func_params_t *params, void *data)
 {
 	printf("  </match>\n");
 	printf(" </device>\n");
@@ -971,29 +1039,41 @@ iterate_camera_list (const int add_comments,
 	CameraAbilitiesList *al;
 	CameraAbilities a;
 	func_params_t params;
+	void *data = NULL;
+	int ret;
 
 	CR (gp_abilities_list_new (&al));
-	CR (gp_abilities_list_load (al, NULL)); /* use NULL context */
-	CR (number_of_cameras = gp_abilities_list_count (al));
+	ret = gp_abilities_list_load (al, NULL); /* use NULL context */
+	if (ret < GP_OK) {
+		gp_abilities_list_free (al);
+		return ret;
+	}
+	number_of_cameras = gp_abilities_list_count (al);
+	if (number_of_cameras < GP_OK) {
+		gp_abilities_list_free (al);
+		return ret;
+	}
 
 	params.add_comments = add_comments;
 	params.number_of_cameras = number_of_cameras;
 	params.argv = argv;
 
 	if (format->begin_func != NULL) {
-		format->begin_func(&params);
+		format->begin_func(&params, &data);
 	}
 
 	if (format->camera_func != NULL) {
 		int i;
 		for (i = 0; i < number_of_cameras; i++) {
-			CR (gp_abilities_list_get_abilities (al, i, &a));
-			format->camera_func(&params, i, number_of_cameras, &a);
+			ret = gp_abilities_list_get_abilities (al, i, &a);
+			if (ret < GP_OK)
+				continue;
+			format->camera_func(&params, i, number_of_cameras, &a, data);
 		}
 	}
 
 	if (format->end_func != NULL) {
-		format->end_func(&params);
+		format->end_func(&params, data);
 	}
 	CR (gp_abilities_list_free (al));
 	return 0;
@@ -1004,78 +1084,70 @@ iterate_camera_list (const int add_comments,
 
 /** list of supported output formats */
 static const output_format_t formats[] = {
-	{name: "human-readable",
-	 descr: "human readable list of cameras",
-	 help: NULL,
-	 paramdescr: NULL,
-	 begin_func: human_begin_func,
-	 camera_func: human_camera_func,
-	 end_func: human_end_func
+	{"human-readable",
+	 "human readable list of cameras",
+	 NULL,
+	 NULL,
+	 human_begin_func,
+	 human_camera_func,
+	 human_end_func
 	},
-	{name: "usb-usermap",
-	 descr: "usb.usermap include file for linux-hotplug",
-	 help: "If no <scriptname> is given, uses the script name "
-	 "\"" GP_USB_HOTPLUG_SCRIPT "\".\n        Put this into /etc/hotplug/usb/<scriptname>.usermap",
-	 paramdescr: "<NAME_OF_HOTPLUG_SCRIPT>",
-	 begin_func: empty_begin_func,
-	 camera_func: hotplug_camera_func,
-	 end_func: empty_end_func
+	{"usb-usermap",
+	 "usb.usermap include file for linux-hotplug",
+	 "If no <scriptname> is given, uses the script name "
+	 "\"" GP_USB_HOTPLUG_SCRIPT "\".\n"
+	 "        Put this into /etc/hotplug/usb/<scriptname>.usermap",
+	 "<NAME_OF_HOTPLUG_SCRIPT>",
+	 hotplug_begin_func,
+	 hotplug_camera_func,
+	 empty_end_func
 	},
-	{name: "hal-fdi",
-	 descr: "fdi file for HAL",
-	 help: "Put it into /usr/share/hal/fdi/information/10freedesktop/10-camera-libgphoto2.fdi",
-	 paramdescr: NULL,
-	 begin_func: fdi_begin_func,
-	 camera_func: fdi_camera_func,
-	 end_func: fdi_end_func
+	{"hal-fdi",
+	 "fdi file for HAL",
+	 "Put it into /usr/share/hal/fdi/information/20thirdparty/10-camera-libgphoto2.fdi",
+	 NULL,
+	 fdi_begin_func,
+	 fdi_camera_func,
+	 fdi_end_func
 	},
-	{name: "hal-fdi-device",
-	 descr: "fdi device file for HAL",
-	 help: "Put it into /usr/share/hal/fdi/information/10freedesktop/10-camera-libgphoto2-device.fdi",
-	 paramdescr: NULL,
-	 begin_func: fdi_device_begin_func,
-	 camera_func: fdi_device_camera_func,
-	 end_func: fdi_device_end_func
+	{"hal-fdi-device",
+	 "fdi device file for HAL",
+	 "Put it into /usr/share/hal/fdi/information/20thirdparty/10-camera-libgphoto2-device.fdi",
+	 NULL,
+	 fdi_device_begin_func,
+	 fdi_device_camera_func,
+	 fdi_device_end_func
 	},
-	{name: "udev-rules",
-	 descr: "udev < 0.98 rules file",
-	 help: "Put it into /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
+	{"udev-rules",
+	 "udev rules file either for pre-0.98 or 0.98 and later",
+	 "Put it into /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
 	 "        or add script to run. This rule files also uses the\n"
 	 "        check-ptp-camera script included in libgphoto2 source. Either put it to\n"
-	 "        /lib/udev/check-ptp-camera or adjust the path in the generated rules file.",
-	 paramdescr: "( <PATH_TO_SCRIPT> | [mode <mode>|owner <owner>|group <group>]* ) ",
-	 begin_func: udev_begin_func, 
-	 camera_func: udev_camera_func,
-	 end_func: udev_end_func
+	 "        /lib/udev/check-ptp-camera or adjust the path in the generated rules file.\n"
+	 "        If you give a script parameter, the mode, owner, group parameters will be ignored.",
+	 "[script <PATH_TO_SCRIPT>|version <version>|mode <mode>|owner <owner>|group <group>]*",
+	 udev_begin_func, 
+	 udev_camera_func,
+	 udev_end_func
 	},
-	{name: "udev-rules-0.98",
-	 descr: "udev >= 0.98 rules file",
-	 help:
-	 "Put it into /etc/udev/rules.d/90-libgphoto2.rules, set file mode, owner, group\n"
-	 "        or add script to run, for udev >= 0.98. This rule files also uses the\n"
-	 "        check-ptp-camera script included in libgphoto2 source. Either put it to\n"
-	 "        /lib/udev/check-ptp-camera or adjust the path in the generated rules file.",
-	 paramdescr: "( <PATH_TO_SCRIPT> | [mode <mode>|owner <owner>|group <group>]* ) ",
-	 begin_func: udev_098_begin_func, 
-	 camera_func: udev_098_camera_func,
-	 end_func: udev_end_func
+	{"idlist",
+	 "list of IDs and names",
+	 "grep for an ID to find the device name",
+	 NULL,
+	 empty_begin_func, 
+	 idlist_camera_func,
+	 empty_end_func
 	},
-	{name: "idlist",
-	 descr: "list of IDs and names",
-	 help: "grep for an ID to find the device name",
-	 paramdescr: NULL,
-	 begin_func: empty_begin_func, 
-	 camera_func: idlist_camera_func,
-	 end_func: empty_end_func
-	},
-	{name: "gp2ddb",
-	 descr: "gphoto2 device database (PRE-ALPHA)",
-	 help: "PRE-ALPHA test stage, do not use for production! Machine parseable.",
-	 paramdescr: NULL,
-	 begin_func: ddb_begin_func,
-	 camera_func: ddb_camera_func,
-	 end_func: ddb_end_func
+#ifdef ENABLED_GP2DDB
+	{"gp2ddb",
+	 "gphoto2 device database (PRE-ALPHA)",
+	 "PRE-ALPHA test stage, do not use for production! Machine parseable.",
+	 NULL,
+	 ddb_begin_func,
+	 ddb_camera_func,
+	 ddb_end_func
 	},
+#endif
 	{NULL, NULL, NULL, NULL, 
 	 NULL, NULL, NULL}
 };
